@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/common.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,8 +38,8 @@ let root2hostname root =
 
 let root2string root =
   match root with
-    (Local, fspath) -> Fspath.toString fspath
-  | (Remote host, fspath) -> "//"^host^"/"^(Fspath.toString fspath)
+    (Local, fspath) -> Fspath.toPrintString fspath
+  | (Remote host, fspath) -> "//"^host^"/"^(Fspath.toPrintString fspath)
 
 (* ------------------------------------------------------------------------- *)
 (*                      Root comparison                                      *)
@@ -50,7 +50,7 @@ let compareRoots x y =
     (Local,fspath1), (Local,fspath2) ->
       (* FIX: This is a path comparison, should it take case
          sensitivity into account ? *)
-      compare (Fspath.toString fspath1) (Fspath.toString fspath2)
+      Fspath.compare fspath1 fspath2
   | (Local,_), (Remote _,_) -> -1
   | (Remote _,_), (Local,_) -> 1
   | (Remote host1, fspath1), (Remote host2, fspath2) ->
@@ -60,7 +60,7 @@ let compareRoots x y =
       if result = 0 then
         (* FIX: This is a path comparison, should it take case
            sensitivity into account ? *)
-        compare (Fspath.toString fspath1) (Fspath.toString fspath2)
+        Fspath.compare fspath1 fspath2
       else
         result
 
@@ -109,31 +109,43 @@ type status =
   | `Created
   | `Unchanged ]
 
-type replicaContent = Fileinfo.typ * status * Props.t * updateItem
+type replicaContent =
+  { typ : Fileinfo.typ;
+    status : status;
+    desc : Props.t;                (* Properties (for the UI) *)
+    ui : updateItem;
+    size : int * Uutil.Filesize.t; (* Number of items and size *)
+    props : Props.t list }         (* Parent properties *)
 
 type direction =
-    Conflict
+    Conflict of string (* The string is the reason of the conflict *)
   | Merge
   | Replica1ToReplica2
   | Replica2ToReplica1
 
 let direction2string = function
-    Conflict -> "conflict"
+    Conflict _ -> "conflict"
   | Merge -> "merge"
   | Replica1ToReplica2 -> "replica1 to replica2"
   | Replica2ToReplica1 -> "replica2 to replica1"
 
-type replicas =
-    Problem of string    (* There was a problem during update detection *)
-  | Different            (* Replicas differ *)
-    of replicaContent    (*   - content of first replica *)
-     * replicaContent    (*   - content of second replica *)
-     * direction ref     (*   - action to take *)
-     * direction         (*   - default action to take *)
+let isConflict = function
+    Conflict _ -> true
+  | _ -> false
 
-type reconItem =
-    {path : Path.t;
-     replicas : replicas}
+type difference =
+  { rc1 : replicaContent;
+    rc2 : replicaContent;
+    errors1 : string list;
+    errors2 : string list;
+    mutable direction : direction;
+    default_direction : direction }
+
+type replicas =
+    Problem of string       (* There was a problem during update detection *)
+  | Different of difference (* Replicas differ *)
+
+type reconItem = {path1 : Path.t; path2 : Path.t; replicas : replicas}
 
 let ucLength = function
     File(desc,_)    -> Props.length desc
@@ -144,8 +156,8 @@ let uiLength = function
     Updates(uc,_) -> ucLength uc
   | _             -> Uutil.Filesize.zero
 
-let riAction (_, s, _, _) (_, s', _, _) =
-  match s, s' with
+let riAction rc rc' =
+  match rc.status, rc'.status with
     `Deleted, _ ->
       `Delete
   | (`Unchanged | `PropsChanged), (`Unchanged | `PropsChanged) ->
@@ -153,19 +165,22 @@ let riAction (_, s, _, _) (_, s', _, _) =
   | _ ->
       `Copy
 
-let rcLength ((_, _, p, _) as rc) rc' =
+let rcLength rc rc' =
   if riAction rc rc' = `SetProps then
     Uutil.Filesize.zero
   else
-    Props.length p
+    snd rc.size
 
 let riLength ri =
   match ri.replicas with
-    Different(rc1, rc2, dir, _) ->
-      begin match !dir with
+    Different {rc1 = {status= `Unchanged | `PropsChanged};
+               rc2 = {status= `Unchanged | `PropsChanged}} ->
+      Uutil.Filesize.zero (* No contents propagated *)
+  | Different {rc1 = rc1; rc2 = rc2; direction = dir} ->
+      begin match dir with
         Replica1ToReplica2 -> rcLength rc1 rc2
       | Replica2ToReplica1 -> rcLength rc2 rc1
-      | Conflict           -> Uutil.Filesize.zero
+      | Conflict _         -> Uutil.Filesize.zero
       | Merge              -> Uutil.Filesize.zero (* underestimate :-*)
       end
   | _ ->
@@ -193,26 +208,32 @@ let fileInfos ui1 ui2 =
 
 let problematic ri =
   match ri.replicas with
-    Problem _ -> true
-  | Different (_,_,d,_) -> (!d = Conflict)
+    Problem _      -> true
+  | Different diff -> isConflict diff.direction
+
+let partiallyProblematic ri =
+  match ri.replicas with
+    Problem _      ->
+      true
+  | Different diff -> 
+     isConflict diff.direction || diff.errors1 <> [] || diff.errors2 <> []
 
 let isDeletion ri =
   match ri.replicas with
-    Different(rc1, rc2, rDir, _) ->
-      (match (!rDir, rc1, rc2) with
-        (Replica1ToReplica2, (`ABSENT, _, _, _), _) -> true
-      | (Replica2ToReplica1, _, (`ABSENT, _, _, _)) -> true
+    Different {rc1 = rc1; rc2 = rc2; direction = rDir} ->
+      (match rDir, rc1.typ, rc2.typ with
+        Replica1ToReplica2, `ABSENT, _ -> true
+      | Replica2ToReplica1, _, `ABSENT -> true
       | _ -> false)
   | _ -> false
 
-let rcType (fi, _, _, _) =
-  Fileinfo.type2string fi
+let rcType rc = Fileinfo.type2string rc.typ
 
 let riFileType ri =
   match ri.replicas with
-    Different(rc1, rc2, dir, _) ->
-      begin match !dir with
+    Different {rc1 = rc1; rc2 = rc2; default_direction = dir} ->
+      begin match dir with
         Replica2ToReplica1 -> rcType rc2
-      | _		           -> rcType rc1
+      | _		   -> rcType rc1
       end
   | _ -> "nonexistent"
