@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/files.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -72,14 +72,52 @@ let processCommitLogs() =
 
 (* ------------------------------------------------------------ *)
 
-let deleteLocal (fspathTo, (pathTo, ui)) =
+let copyOnConflict = Prefs.createBool "copyonconflict" false
+  "!keep copies of conflicting files"
+  "When this flag is set, Unison will make a copy of files that would \
+   otherwise be overwritten or deleted in case of conflicting changes, \
+   and more generally whenever the default behavior is overriden. \
+   This makes it possible to automatically resolve conflicts in a \
+   fairly safe way when synchronizing continuously, in combination \
+   with the \\verb|-repeat watch| and \\verb|-prefer newer| preferences."
+
+let prepareCopy workingDir path notDefault =
+  if notDefault && Prefs.read copyOnConflict then begin
+    let tmpPath = Os.tempPath workingDir path in
+    Copy.recursively workingDir path workingDir tmpPath;
+    Some (workingDir, path, tmpPath)
+  end else
+    None
+
+let finishCopy copyInfo =
+  match copyInfo with
+    Some (workingDir, path, tmpPath) ->
+      let tm = Unix.localtime (Unix.gettimeofday ()) in
+      let rec copyPath n =
+        let p =
+          Path.addToFinalName path
+            (Format.sprintf " (copy: conflict%s on %04d-%02d-%02d)"
+               (if n = 0 then "" else " #" ^ string_of_int n)
+               (tm.Unix.tm_year + 1900) (tm.Unix.tm_mon + 1) tm.Unix.tm_mday)
+        in
+        if Os.exists workingDir p then copyPath (n + 1) else p
+      in
+      Os.rename "keepCopy" workingDir tmpPath workingDir (copyPath 0)
+  | None ->
+      ()
+
+(* ------------------------------------------------------------ *)
+
+let deleteLocal (fspathTo, (pathTo, ui, notDefault)) =
   debug (fun () ->
      Util.msg "deleteLocal [%s] (None, %s)\n"
        (Fspath.toDebugString fspathTo) (Path.toString pathTo));
   let localPathTo = Update.translatePathLocal fspathTo pathTo in
+  let copyInfo = prepareCopy fspathTo localPathTo notDefault in
   (* Make sure the target is unchanged first *)
   (* (There is an unavoidable race condition here.) *)
   let prevArch = Update.checkNoUpdates fspathTo localPathTo ui in
+  finishCopy copyInfo;
   Stasher.backup fspathTo localPathTo `AndRemove prevArch;
   (* Archive update must be done last *)
   Update.replaceArchiveLocal fspathTo localPathTo Update.NoArchive;
@@ -87,8 +125,8 @@ let deleteLocal (fspathTo, (pathTo, ui)) =
 
 let deleteOnRoot = Remote.registerRootCmd "delete" deleteLocal
 
-let delete rootFrom pathFrom rootTo pathTo ui =
-  deleteOnRoot rootTo (pathTo, ui) >>= fun _ ->
+let delete rootFrom pathFrom rootTo pathTo ui notDefault =
+  deleteOnRoot rootTo (pathTo, ui, notDefault) >>= fun _ ->
   Update.replaceArchive rootFrom pathFrom Update.NoArchive
 
 (* ------------------------------------------------------------ *)
@@ -268,10 +306,13 @@ let performRename fspathTo localPathTo workingDir pathFrom pathTo prevArch =
    temp file into place, but remain able to roll back if something fails
    either locally or on the other side. *)
 let renameLocal
-      (fspathTo, (localPathTo, workingDir, pathFrom, pathTo, ui, archOpt)) =
+      (fspathTo,
+       (localPathTo, workingDir, pathFrom, pathTo, ui, archOpt, notDefault)) =
+  let copyInfo = prepareCopy workingDir pathTo notDefault in
   (* Make sure the target is unchanged, then do the rename.
      (Note that there is an unavoidable race condition here...) *)
   let prevArch = Update.checkNoUpdates fspathTo localPathTo ui in
+  finishCopy copyInfo;
   performRename fspathTo localPathTo workingDir pathFrom pathTo prevArch;
   begin match archOpt with
     Some archTo -> Stasher.stashCurrentVersion fspathTo localPathTo None;
@@ -285,12 +326,13 @@ let renameLocal
 
 let renameOnHost = Remote.registerRootCmd "rename" renameLocal
 
-let rename root localPath workingDir pathOld pathNew ui archOpt =
+let rename root localPath workingDir pathOld pathNew ui archOpt notDefault =
   debug (fun() ->
     Util.msg "rename(root=%s, pathOld=%s, pathNew=%s)\n"
       (root2string root)
       (Path.toString pathOld) (Path.toString pathNew));
-  renameOnHost root (localPath, workingDir, pathOld, pathNew, ui, archOpt)
+  renameOnHost root
+    (localPath, workingDir, pathOld, pathNew, ui, archOpt, notDefault)
 
 (* ------------------------------------------------------------ *)
 
@@ -431,6 +473,7 @@ let copy
                              this updateItem still describes the current
                              state of the target replica) *)
       propsTo             (* the properties of the parent directories *)
+      notDefault          (* [true] if not Unison's default action *)
       id =                (* for progress display *)
   debug (fun() ->
     Util.msg
@@ -471,19 +514,19 @@ let copy
     Lwt.catch
       (fun () ->
          match f with
-           Update.ArchiveFile (desc, dig, stamp, ress) ->
+           Update.ArchiveFile (desc, fp, stamp, ress) ->
              Lwt_util.run_in_region copyReg 1 (fun () ->
                Abort.check id;
                let stmp =
                  if Update.useFastChecking () then Some stamp else None in
                Copy.file
                  rootFrom pFrom rootTo workingDir pTo realPTo
-                 update desc dig stmp ress id
+                 update desc fp stmp ress id
                  >>= fun info ->
                let ress' = Osx.stamp info.Fileinfo.osX in
                Lwt.return
                  (Update.ArchiveFile (Props.override info.Fileinfo.desc desc,
-                                      dig, Fileinfo.stamp info, ress'),
+                                      fp, Fileinfo.stamp info, ress'),
                   []))
          | Update.ArchiveSymlink l ->
              Lwt_util.run_in_region copyReg 1 (fun () ->
@@ -564,7 +607,7 @@ let copy
     (* Rename the files to their final location and then update the
        archive on the destination replica *)
     rename rootTo localPathTo workingDir tempPathTo realPathTo uiTo
-      (Some archTo) >>= fun () ->
+      (Some archTo) notDefault >>= fun () ->
     (* Update the archive on the source replica
        FIX: we could reuse localArch if rootFrom is the same as rootLocal *)
     updateSourceArchive rootFrom (localPathFrom, uiFrom, errPaths) >>= fun () ->
@@ -733,7 +776,8 @@ let formatMergeCmd p f1 f2 backup out1 out2 outarch =
   let cooked = Util.replacesubstring cooked "NEW2"     out2 in
   let cooked = Util.replacesubstring cooked "NEWARCH"  outarch in
   let cooked = Util.replacesubstring cooked "NEW" out1 in
-  let cooked = Util.replacesubstring cooked "PATH" (Path.toString p) in
+  let cooked = Util.replacesubstring cooked "PATH"
+		(Uutil.quotes (Path.toString p)) in
   cooked
     
 let copyBack fspathFrom pathFrom rootTo pathTo propsTo uiTo id =
@@ -747,7 +791,7 @@ let copyBack fspathFrom pathFrom rootTo pathTo propsTo uiTo id =
     (Local, fspathFrom) pathFrom rootTo workingDirForCopy tempPathTo realPathTo
     `Copy newprops fp None stamp id >>= fun info ->
   rename rootTo localPathTo workingDirForCopy tempPathTo realPathTo
-    uiTo None)
+    uiTo None false)
     
 let keeptempfilesaftermerge =   
   Prefs.createBool
@@ -822,14 +866,14 @@ let merge root1 path1 ui1 root2 path2 ui2 id showMergeFn =
       (* retrieve the archive for this file, if any *)
       let arch =
 	match ui1, ui2 with
-	| Updates (_, Previous (_,_,dig,_)), Updates (_, Previous (_,_,dig2,_)) ->
-	    if dig = dig2 then
-	      Stasher.getRecentVersion fspath1 localPath1 dig 
+	| Updates (_, Previous (_,_,fp,_)), Updates (_, Previous (_,_,fp2,_)) ->
+	    if fp = fp2 then
+	      Stasher.getRecentVersion fspath1 localPath1 fp 
 	    else
 	      assert false
-	| NoUpdates, Updates(_, Previous (_,_,dig,_))
-	| Updates(_, Previous (_,_,dig,_)), NoUpdates -> 
-	    Stasher.getRecentVersion fspath1 localPath1 dig
+	| NoUpdates, Updates(_, Previous (_,_,fp,_))
+	| Updates(_, Previous (_,_,fp,_)), NoUpdates -> 
+	    Stasher.getRecentVersion fspath1 localPath1 fp
 	| Updates (_, New), Updates(_, New) 
 	| Updates (_, New), NoUpdates
 	| NoUpdates, Updates (_, New) ->
@@ -860,9 +904,9 @@ let merge root1 path1 ui1 root2 path2 ui2 id showMergeFn =
       Os.delete workingDirForMerge newarch;
       let info1 = Fileinfo.get false workingDirForMerge working1 in
       (* FIX: Why split out the parts of the pair?  Why is it not abstract anyway??? *)
-      let dig1 = Os.fingerprint workingDirForMerge working1 info1 in
+      let fp1 = Os.fingerprint workingDirForMerge working1 info1 in
       let info2 = Fileinfo.get false workingDirForMerge working2 in
-      let dig2 = Os.fingerprint workingDirForMerge working2 info2 in
+      let fp2 = Os.fingerprint workingDirForMerge working2 info2 in
       let cmd = formatMergeCmd
           path1
           (Fspath.quotes (Fspath.concat workingDirForMerge working1))
@@ -910,9 +954,9 @@ let merge root1 path1 ui1 root2 path2 ui2 id showMergeFn =
 	  say (fun () -> Util.msg "Two outputs detected \n");
         let info1 = Fileinfo.get false workingDirForMerge new1 in
         let info2 = Fileinfo.get false workingDirForMerge new2 in
-        let dig1' = Os.fingerprint workingDirForMerge new1 info1 in
-        let dig2' = Os.fingerprint workingDirForMerge new2 info2 in
-        if dig1'=dig2' then begin
+        let fp1' = Os.fingerprint workingDirForMerge new1 info1 in
+        let fp2' = Os.fingerprint workingDirForMerge new2 info2 in
+        if fp1'=fp2' then begin
           debug (fun () -> Util.msg "Two outputs equal => update the archive\n");
           copy [(new1,working1); (new2,working2); (new1,workingarch)];
 	end else
@@ -950,18 +994,18 @@ let merge root1 path1 ui1 root2 path2 ui2 id showMergeFn =
         if working1_still_exists && working2_still_exists then begin
           say (fun () -> Util.msg "No output from merge cmd and both original files are still present\n");
           let info1' = Fileinfo.get false workingDirForMerge working1 in
-          let dig1' = Os.fingerprint workingDirForMerge working1 info1' in
+          let fp1' = Os.fingerprint workingDirForMerge working1 info1' in
           let info2' = Fileinfo.get false workingDirForMerge working2 in
-          let dig2' = Os.fingerprint workingDirForMerge working2 info2' in
-          if dig1 = dig1' && dig2 = dig2' then
+          let fp2' = Os.fingerprint workingDirForMerge working2 info2' in
+          if fp1 = fp1' && fp2 = fp2' then
             raise (Util.Transient "Merge program didn't change either temp file");
-          if dig1' = dig2' then begin
+          if fp1' = fp2' then begin
             say (fun () -> Util.msg "Merge program made files equal\n");
             copy [(working1,workingarch)];
-          end else if dig2 = dig2' then begin
+          end else if fp2 = fp2' then begin
             say (fun () -> Util.msg "Merge program changed just first input\n");
             copy [(working1,working2);(working1,workingarch)]
-          end else if dig1 = dig1' then begin
+          end else if fp1 = fp1' then begin
             say (fun () -> Util.msg "Merge program changed just second input\n");
             copy [(working2,working1);(working2,workingarch)]
           end else
@@ -1016,11 +1060,11 @@ let merge root1 path1 ui1 root2 path2 ui2 id showMergeFn =
              Util.msg "Warning: 'backupcurrent' is not set for path %s\n" (Path.toString path1);
            Stasher.stashCurrentVersion workingDirForMerge localPath1 (Some workingarch);
            let infoarch = Fileinfo.get false workingDirForMerge workingarch in
-           let dig = Os.fingerprint arch_fspath Path.empty infoarch in
-           debug (fun () -> Util.msg "New digest is %s\n" (Os.fullfingerprint_to_string dig));
+           let fp = Os.fingerprint arch_fspath Path.empty infoarch in
+           debug (fun () -> Util.msg "New fingerprint is %s\n" (Os.fullfingerprint_to_string fp));
            let new_archive_entry =
              Update.ArchiveFile
-               (Props.get (Fs.stat arch_fspath) infoarch.osX, dig,
+               (Props.get (Fs.stat arch_fspath) infoarch.osX, fp,
                 Fileinfo.stamp (Fileinfo.get true arch_fspath Path.empty),
                 Osx.stamp infoarch.osX) in
            Update.replaceArchive root1 path1 new_archive_entry >>= fun _ ->
