@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/fingerprint.mli *)
-(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
+(* Copyright 1999-2015, Benjamin C. Pierce (see COPYING for details) *)
 
 type t
 
@@ -17,3 +17,9 @@ val dummy : t
 
 val hash : t -> int
 val equal : t -> t -> bool
+
+(* A pseudo-fingerprint has the same type as a real one (so it can
+   be stored in the archive, etc.), but it is computed just from the
+   size of the file, ignoring the contents *)
+val pseudo : Path.local -> Uutil.Filesize.t -> t
+val ispseudo : t -> bool
