@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/globals.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -302,5 +302,5 @@ let fatFilesystem =
       do not attempt to synchronize symbolic links ({\\tt links = false}); \
       ignore inode number changes when detecting updates \
       ({\\tt ignoreinodenumbers = true}).  \
-      Any of these change can be overridden by explicitely setting \
+      Any of these change can be overridden by explicitly setting \
       the corresponding preference in the profile.")
