@@ -1,17 +1,18 @@
 (* Unison file synchronizer: src/os.mli *)
-(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
+(* Copyright 1999-2015, Benjamin C. Pierce (see COPYING for details) *)
 
-val myCanonicalHostName : string
+val myCanonicalHostName : unit -> string
 
 val tempPath : ?fresh:bool -> Fspath.t -> Path.local -> Path.local
 val tempFilePrefix : string
+val isTempFile : string -> bool
 val includeInTempNames : string -> unit
 
 val exists : Fspath.t -> Path.local -> bool
 
 val createUnisonDir : unit -> unit
-val fileInUnisonDir : string -> Fspath.t
-val unisonDir : Fspath.t
+val fileInUnisonDir : string -> System.fspath
+val unisonDir : System.fspath
 
 val childrenOf : Fspath.t -> Path.local -> Name.t list
 val readLink : Fspath.t -> Path.local -> string
@@ -31,6 +32,8 @@ type fullfingerprint
 val fullfingerprint_to_string : fullfingerprint -> string
 val reasonForFingerprintMismatch : fullfingerprint -> fullfingerprint -> string
 val fullfingerprint_dummy : fullfingerprint
+val fullfingerprintHash : fullfingerprint -> int
+val fullfingerprintEqual : fullfingerprint -> fullfingerprint -> bool
 
 (* Use this function if the file may change during fingerprinting *)
 val safeFingerprint :
@@ -44,15 +47,11 @@ val fingerprint :
   Fileinfo.t ->             (* old fileinfo *)
   fullfingerprint           (* current fingerprint *)
 
-(* Versions of system calls that will restart when interrupted by
-   signal handling *)
-val accept : Unix.file_descr -> (Unix.file_descr * Unix.sockaddr)
+val pseudoFingerprint :
+  Path.local             -> (* path of file to "fingerprint" *)
+  Uutil.Filesize.t       -> (* size of file to "fingerprint" *)
+  fullfingerprint           (* pseudo-fingerprint of this file (containing just
+                               the file's length and path) *)
 
-(* Called during program initialization to resolve a circular dependency
-   between this module and Xferhints *)
-val initializeXferFunctions : 
-    (Fspath.t * Path.local -> unit) -> 
-    ((Fspath.t * Path.local) -> (Fspath.t * Path.local) -> unit) ->
-    unit
-
-val quotes : string -> string
+val isPseudoFingerprint :
+  fullfingerprint -> bool
