@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/path.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -188,6 +188,14 @@ let toDebugString path = String.concat " / " (toStringList path)
 
 let addSuffixToFinalName path suffix = path ^ suffix
 
+let addToFinalName path suffix =
+  let l = String.length path in
+  assert (l > 0);
+  let i = try String.rindex path '/' with Not_found -> -1 in
+  let j = try String.rindex path '.' with Not_found -> -1 in
+  let j = if j <= i then l else j in
+  String.sub path 0 j ^ suffix ^ String.sub path j (l - j)
+
 let addPrefixToFinalName path prefix =
   try
     let i = String.rindex path pathSeparatorChar + 1 in
@@ -202,10 +210,6 @@ let addPrefixToFinalName path prefix =
     assert (not (isEmpty path));
     prefix ^ path
 
-(* No need to perform case normalization on local paths *)
-let hash p = Hashtbl.hash p
-let equal (p1 : local) (p2 : local) = p1 = p2
-
 (* Pref controlling whether symlinks are followed. *)
 let followPred = Pred.create ~advanced:true "follow"
     ("Including the preference \\texttt{-follow \\ARG{pathspec}} causes Unison to \
