@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/path.mli *)
-(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
+(* Copyright 1999-2015, Benjamin C. Pierce (see COPYING for details) *)
 
 (* Abstract type of relative pathnames *)
 type 'a path
@@ -29,6 +29,8 @@ val toDebugString : local -> string
 
 val addSuffixToFinalName : local -> string -> local
 val addPrefixToFinalName : local -> string -> local
+val addToFinalName : local -> string -> local
+  (* Add to the final name, but before any file extension. *)
 
 val compare : t -> t -> int
 
