@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/remote.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -896,7 +896,10 @@ let registerStreamCmd
            Lwt_util.run_in_region streamReg 1
              (fun () -> sender (fun v -> client conn id v)))
         (fun v -> ping conn id >>= fun () -> Lwt.return v)
-        (fun e -> ping conn id >>= fun () -> Lwt.fail e)
+        (fun e ->
+           debugE (fun () ->
+             Util.msg "Pinging remote end after streaming error\n");
+           ping conn id >>= fun () -> Lwt.fail e)
     end
 
 let commandAvailable =
@@ -1168,7 +1171,7 @@ let canonizeLocally s unicode =
 let canonizeOnServer =
   registerServerCmd "canonizeOnServer"
     (fun _ (s, unicode) ->
-       Lwt.return (Os.myCanonicalHostName, canonizeLocally s unicode))
+       Lwt.return (Os.myCanonicalHostName (), canonizeLocally s unicode))
 
 let canonize clroot = (* connection for clroot must have been set up already *)
   match clroot with
