@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/test.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -214,7 +214,7 @@ let displayRis ris =
 
 let sync ?(verbose=false) () = 
   let (reconItemList, _, _) =
-    Recon.reconcileAll (Update.findUpdates()) in
+    Recon.reconcileAll (Update.findUpdates None) in
   if verbose then begin
     Util.msg "Sync result:\n";
     displayRis reconItemList
@@ -339,6 +339,39 @@ let test() =
       check "6" BACKUP2 (Dir [("test.txt", File "1")]);
     );
 
+  if bothRootsLocal then 
+    runtest "fastercheckUNSAFE 1" ["fastercheckUNSAFE = true"] (fun() -> 
+      put R1 (Dir []); put R2 (Dir []); sync();
+      (* Create a file on both sides with different contents *)
+      put R1 (Dir ["x", File "foo"]); 
+      put R2 (Dir ["x", File "bar"]); sync();
+      check "1a" R1 (Dir ["x", File "foo"]);
+      check "1b" R2 (Dir ["x", File "bar"]);
+      (* Change contents on one side and see that we do NOT get a conflict (!) *)
+      put R1 (Dir ["x", File "newcontents"]); sync();
+      check "2a" R1 (Dir ["x", File "newcontents"]);
+      check "2b" R2 (Dir ["x", File "newcontents"]);
+
+      (* Start again *)
+      put R1 (Dir []); put R2 (Dir []); sync();
+      (* Create a file on both sides with different contents *)
+      put R1 (Dir ["x", File "foo"]); 
+      put R2 (Dir ["x", File "bar"]); sync();
+      (* Change contents without changing size and check that change is propagated *)
+      put R1 (Dir ["x", File "f00"]); sync();
+      check "3a" R1 (Dir ["x", File "f00"]);
+      check "3b" R2 (Dir ["x", File "f00"]);
+
+      (* Start again *)
+      put R1 (Dir []); put R2 (Dir []); sync();
+      (* Create a new file on one side only *)
+      put R1 (Dir ["x", File "foo"]); sync();
+      (* Check that change is propagated *)
+      check "4" R2 (Dir ["x", File "foo"]);
+    );
+
+  (raise (Util.Fatal "Skipping some tests -- remove me!\n") : unit); 
+
   if bothRootsLocal then 
     runtest "backups 1 (local)" ["backup = Name *"] (fun() -> 
       put R1 (Dir []); put R2 (Dir []); sync();
