@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/transfer.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -103,7 +103,7 @@ let queueSizeFS = Uutil.Filesize.ofInt queueSize
 type tokenQueue =
   { mutable data : Bytearray.t;  (* the queued tokens *)
     mutable previous : [`Str of int | `Block of int | `None];
-                                 (* some informations about the
+                                 (* some information about the
                                     previous token *)
     mutable pos : int;           (* head of the queue *)
     mutable prog : int;          (* the size of the data they represent *)
@@ -147,8 +147,12 @@ let flushQueue q showProgress transmit cond =
     return ()
 
 let pushEOF q showProgress transmit =
+  if Trace.enabled "rsynctoken" then
+    debugToken (fun() ->
+      Util.msg "pushing EOF (pos:%d/%d)\n" q.pos queueSize);
   flushQueue q showProgress transmit
     (q.pos + 1 > queueSize) >>= (fun () ->
+  assert (q.pos < queueSize);
   q.data.{q.pos} <- 'E';
   q.pos <- q.pos + 1;
   q.previous <- `None;
@@ -156,7 +160,11 @@ let pushEOF q showProgress transmit =
 
 let rec pushString q id transmit s pos len =
   flushQueue q id transmit (q.pos + len + 3 > queueSize) >>= fun () ->
+  if Trace.enabled "rsynctoken" then
+    debugToken (fun() ->
+      Util.msg "pushing string (pos:%d/%d len:%d)\n" q.pos queueSize len);
   let l = min len (queueSize - q.pos - 3) in
+  assert (l > 0);
   q.data.{q.pos} <- 'S';
   encodeInt2 q.data (q.pos + 1) l;
   Bytearray.blit_from_string s pos q.data (q.pos + 3) l;
@@ -169,8 +177,13 @@ let rec pushString q id transmit s pos len =
     return ()
 
 let growString q id transmit len' s pos len =
+  if Trace.enabled "rsynctoken" then
+    debugToken (fun() ->
+      Util.msg "growing string (pos:%d/%d len:%d+%d)\n"
+        q.pos queueSize len' len);
   let l = min (queueSize - q.pos) len in
   Bytearray.blit_from_string s pos q.data q.pos l;
+  assert (q.pos - len' - 3 >= 0);
   assert (q.data.{q.pos - len' - 3} = 'S');
   assert (decodeInt2 q.data (q.pos - len' - 2) = len');
   let len'' = len' + l in
@@ -185,6 +198,10 @@ let growString q id transmit len' s pos len =
 
 let pushBlock q id transmit pos =
   flushQueue q id transmit (q.pos + 5 > queueSize) >>= (fun () ->
+  if Trace.enabled "rsynctoken" then
+    debugToken (fun() ->
+      Util.msg "pushing block (pos:%d/%d)\n" q.pos queueSize);
+  assert (q.pos + 5 <= queueSize);
   q.data.{q.pos} <- 'B';
   encodeInt3 q.data (q.pos + 1) pos;
   encodeInt1 q.data (q.pos + 4) 1;
@@ -194,6 +211,10 @@ let pushBlock q id transmit pos =
   return ())
 
 let growBlock q id transmit pos =
+  if Trace.enabled "rsynctoken" then
+    debugToken (fun() ->
+      Util.msg "growing blocks (pos:%d/%d)\n" q.pos queueSize);
+  assert (q.pos >= 5);
   let count = decodeInt1 q.data (q.pos - 1) in
   assert (q.data.{q.pos - 5} = 'B');
   assert (decodeInt3 q.data (q.pos - 4) + count = pos);
@@ -243,15 +264,20 @@ let send infd length showProgress transmit =
   let buf = String.create bufSz in
   let q = makeQueue 0 in
   let rec sendSlice length =
-    let count =
-      reallyRead infd buf 0
-        (if length > bufSzFS then bufSz else Uutil.Filesize.toInt length) in
-    queueToken q showProgress transmit (STRING (buf, 0, count)) >>= (fun () ->
-    let length = Uutil.Filesize.sub length (Uutil.Filesize.ofInt count) in
-    if count = bufSz && length > Uutil.Filesize.zero then
-      sendSlice length
-    else
-      return ())
+    if length > Uutil.Filesize.zero then begin
+      let count =
+        reallyRead infd buf 0
+          (if length > bufSzFS then bufSz else Uutil.Filesize.toInt length) in
+      if count = 0 then
+        Lwt.return ()
+      else begin
+        queueToken q showProgress transmit (STRING (buf, 0, count))
+          >>= fun () ->
+        let length = Uutil.Filesize.sub length (Uutil.Filesize.ofInt count) in
+        sendSlice length
+      end
+    end else
+      Lwt.return ()
   in
   sendSlice length >>= (fun () ->
   queueToken q showProgress transmit EOF >>= (fun () ->
@@ -397,9 +423,23 @@ struct
       blockIter infd addBlock blockSize (min blockCount (256*256*256)) in
     debugLog (fun() -> Util.msg "%d blocks\n" count);
     Trace.showTimer timer;
-    ({ blockSize = blockSize; blockCount = count; checksumSize = csSize;
-       weakChecksum = weakCs; strongChecksum = strongCs },
-     blockSize)
+    let sigs =
+      { blockSize = blockSize; blockCount = count; checksumSize = csSize;
+        weakChecksum = weakCs; strongChecksum = strongCs } in
+    if
+      sigs.blockCount > Bigarray.Array1.dim sigs.weakChecksum ||
+      sigs.blockCount * sigs.checksumSize >
+      Bigarray.Array1.dim sigs.strongChecksum
+    then
+      raise
+        (Util.Transient
+           (Format.sprintf
+              "Internal error during rsync transfer (preprocess), \
+               please report: %d %d - %d %d"
+              sigs.blockCount (Bigarray.Array1.dim sigs.weakChecksum)
+              (sigs.blockCount * sigs.checksumSize)
+              (Bigarray.Array1.dim sigs.strongChecksum)));
+    (sigs, blockSize)
 
   (* Expected size of the [rsync_block_info] datastructure (in KiB). *)
   let memoryFootprint srcLength dstLength =
@@ -495,7 +535,8 @@ struct
   (* Given a key, retrieve the corresponding entry in the table *)
   let findEntry hashTable hashTableLength checksum :
       (int * Checksum.t) list =
-    Array.unsafe_get hashTable ((hash checksum) land (hashTableLength - 1))
+    let i = (hash checksum) land (hashTableLength - 1) in
+    hashTable.(i)
 
   let sigFilter hashTableLength signatures =
     let len = hashTableLength lsl 2 in
@@ -588,6 +629,19 @@ struct
   (* Compress the file using the algorithm described in the header *)
   let rsyncCompress sigs infd srcLength showProgress transmit =
     debug (fun() -> Util.msg "compressing\n");
+    if
+      sigs.blockCount > Bigarray.Array1.dim sigs.weakChecksum ||
+      sigs.blockCount * sigs.checksumSize >
+      Bigarray.Array1.dim sigs.strongChecksum
+    then
+      raise
+        (Util.Transient
+           (Format.sprintf
+              "Internal error during rsync transfer (compression), \
+               please report: %d %d - %d %d"
+              sigs.blockCount (Bigarray.Array1.dim sigs.weakChecksum)
+              (sigs.blockCount * sigs.checksumSize)
+              (Bigarray.Array1.dim sigs.strongChecksum)));
     let blockSize = sigs.blockSize in
     let comprBufSize = (2 * blockSize + 8191) land (-8192) in
     let comprBufSizeFS = Uutil.Filesize.ofInt comprBufSize in
@@ -638,12 +692,23 @@ struct
     let rec fingerprintMatchRec checksums pos fp i =
       let i = i - 1 in
       i < 0 ||
-      (String.unsafe_get fp i = checksums.{pos + i} &&
+      (fp.[i] = checksums.{pos + i} &&
        fingerprintMatchRec checksums pos fp i)
     in
     let fingerprintMatch k fp =
-      fingerprintMatchRec sigs.strongChecksum (k * sigs.checksumSize)
-        fp sigs.checksumSize
+      let pos = k * sigs.checksumSize in
+      (*FIX: temporary debugging code... *)
+      if
+        pos + sigs.checksumSize > Bigarray.Array1.dim sigs.strongChecksum
+      then
+        raise
+          (Util.Transient
+             (Format.sprintf "Internal error during rsync transfer, \
+                              please report: \
+                              k:%d/%d pos:%d csSize:%d dim:%d"
+                k sigs.blockCount pos sigs.checksumSize
+                (Bigarray.Array1.dim sigs.strongChecksum)));
+      fingerprintMatchRec sigs.strongChecksum pos fp sigs.checksumSize
     in
 
     (* Create the compression buffer *)
