@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/uicommon.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -93,21 +93,9 @@ let repeat =
     ("Setting this preference causes the text-mode interface to synchronize "
      ^ "repeatedly, rather than doing it just once and stopping.  If the "
      ^ "argument is a number, Unison will pause for that many seconds before "
-     ^ "beginning again.")
-
-(*   ^ "If the argument is a path, Unison will wait for the "
-     ^ "file at this path---called a {\\em changelog}---to "
-     ^ "be modified (on either the client or the server "
-     ^ "machine), read the contents of the changelog (which should be a newline-"
-     ^ "separated list of paths) on both client and server, "
-     ^ "combine the results, "
-     ^ "and start again, using the list of paths read from the changelogs as the "
-     ^ " '-path' preference for the new run.  The idea is that an external "
-     ^ "process will watch the filesystem and, when it thinks something may have "
-     ^ "changed, write the changed pathname to its local changelog where Unison "
-     ^ "will find it the next time it looks.  If the changelogs have not been "
-     ^ "modified, Unison will wait, checking them again every few seconds."
-*)
+     ^ "beginning again. When the argument is \\verb|watch|, Unison relies on "
+     ^ "an external file monitoring process to synchronize whenever a change "
+     ^ "happens.")
 
 let retry =
   Prefs.createInt "retry" 0
@@ -120,9 +108,9 @@ let retry =
 
 let confirmmerge =
   Prefs.createBool "confirmmerge" false
-    "!ask for confirmation before commiting results of a merge"
+    "!ask for confirmation before committing results of a merge"
     ("Setting this preference causes both the text and graphical interfaces"
-     ^ " to ask the user if the results of a merge command may be commited "
+     ^ " to ask the user if the results of a merge command may be committed "
      ^ " to the replica or not. Since the merge command works on temporary files,"
      ^ " the user can then cancel all the effects of applying the merge if it"
      ^ " turns out that the result is not satisfactory.  In "
@@ -279,7 +267,7 @@ type action = AError | ASkip of bool | ALtoR of bool | ARtoL of bool | AMerge
 
 let direction2action partial dir =
   match dir with
-    Conflict           -> ASkip partial
+    Conflict _         -> ASkip partial
   | Replica1ToReplica2 -> ALtoR partial
   | Replica2ToReplica1 -> ARtoL partial
   | Merge              -> AMerge
@@ -309,20 +297,24 @@ let reconItem2string oldPath theRI status =
   let (r1, action, r2, path) = reconItem2stringList oldPath theRI in
   Format.sprintf "%s %s %s %s %s" r1 (action2niceString action) r2 status path
 
-let exn2string = function
-    Sys.Break      -> "Terminated!"
-  | Util.Fatal(s)  -> Printf.sprintf "Fatal error: %s" s
-  | Util.Transient(s) -> Printf.sprintf "Error: %s" s
-  | Unix.Unix_error (err, fun_name, arg) ->
-      Printf.sprintf "Uncaught unix error: %s failed%s: %s%s"
-        fun_name
-        (if String.length arg > 0 then Format.sprintf " on \"%s\"" arg else "")
-        (Unix.error_message err)
-        (match err with
-           Unix.EUNKNOWNERR n -> Format.sprintf " (code %d)" n
-         | _                  -> "")
-  | Invalid_argument s -> Printf.sprintf "Invalid argument: %s" s
-  | other -> Printf.sprintf "Uncaught exception %s" (Printexc.to_string other)
+let exn2string e =
+  match e with
+     Sys.Break      -> "Terminated!"
+   | Util.Fatal(s)  -> Printf.sprintf "Fatal error: %s" s
+   | Util.Transient(s) -> Printf.sprintf "Error: %s" s
+   | Unix.Unix_error (err, fun_name, arg) ->
+       Printf.sprintf "Uncaught unix error: %s failed%s: %s%s\n%s"
+         fun_name
+         (if String.length arg > 0 then Format.sprintf " on \"%s\"" arg else "")
+         (Unix.error_message err)
+         (match err with
+            Unix.EUNKNOWNERR n -> Format.sprintf " (code %d)" n
+          | _                  -> "")
+         (Printexc.get_backtrace ())
+   | Invalid_argument s ->
+       Printf.sprintf "Invalid argument: %s\n%s" s (Printexc.get_backtrace ())
+   | other -> Printf.sprintf "Uncaught exception %s\n%s"
+       (Printexc.to_string other) (Printexc.get_backtrace ())
 
 (* precondition: uc = File (Updates(_, ..) on both sides *)
 let showDiffs ri printer errprinter id =
