@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/uitext.ml *)
-(* Copyright 1999-2009, Benjamin C. Pierce 
+(* Copyright 1999-2015, Benjamin C. Pierce 
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)
 
-
 open Common
 open Lwt
 
@@ -49,8 +48,7 @@ let silent =
 
 let cbreakMode = ref None
 
-(* FIX: this may also work with Cygwin, but someone needs to try it... *)
-let supportSignals = Util.osType = `Unix (*|| Util.isCygwin*)
+let supportSignals = Util.osType = `Unix || Util.isCygwin
 
 let rawTerminal () =
   match !cbreakMode with
@@ -220,7 +218,7 @@ let displayri ri =
   match ri.replicas with
     Problem _ ->
       alwaysDisplay s
-  | Different {direction = d} when d=Conflict ->
+  | Different {direction = d} when isConflict d ->
       alwaysDisplay s
   | _ ->
       display s
@@ -256,7 +254,7 @@ let interact rilist =
         match ri.replicas with
           Problem s -> display "\n"; display s; display "\n"; next()
         | Different ({rc1 = rc1; rc2 = rc2; direction = dir} as diff) ->
-            if Prefs.read Uicommon.auto && dir<>Conflict then begin
+            if Prefs.read Uicommon.auto && not (isConflict dir) then begin
               display "\n"; next()
             end else
               let (descr, descl) =
@@ -273,14 +271,14 @@ let interact rilist =
               end;
               selectAction
                 (if Prefs.read Globals.batch then Some " " else None)
-                [((if dir=Conflict && not (Prefs.read Globals.batch)
+                [((if (isConflict dir) && not (Prefs.read Globals.batch)
                      then ["f"]  (* Offer no default behavior if we've got
                                     a conflict and we're in interactive mode *)
                      else ["";"f";" "]),
                   ("follow " ^ Uutil.myName ^ "'s recommendation (if any)"),
                   fun ()->
                     newLine ();
-                    if dir = Conflict && not (Prefs.read Globals.batch)
+                    if (isConflict dir) && not (Prefs.read Globals.batch)
                     then begin
                       display "No default action [type '?' for help]\n";
                       repeat()
@@ -362,7 +360,7 @@ let interact rilist =
                  (["/"],
                   ("skip"),
                   (fun () ->
-                    diff.direction <- Conflict;
+                    if not (isConflict dir) then diff.direction <- Conflict "skip requested";
                     redisplayri();
                     next()));
                  ([">";"."],
@@ -580,9 +578,12 @@ let rec interactAndPropagateChanges reconItemList
     if skipped>0 then
       Safelist.iter
         (fun ri ->
-        if problematic ri then
-          alwaysDisplayAndLog
-            ("  skipped: " ^ (Path.toString ri.path1)))
+         match ri.replicas with
+           Problem r
+         | Different {rc1 = _; rc2 = _; direction = Conflict r; default_direction = _} ->
+            alwaysDisplayAndLog (Printf.sprintf "  skipped: %s (%s)"
+                                                (Path.toString ri.path1) r)
+         | _ -> ())
         newReconItemList;
     if partials>0 then
       Safelist.iter
@@ -606,6 +607,25 @@ let rec interactAndPropagateChanges reconItemList
     if !Update.foundArchives && Prefs.read Uicommon.repeat = "" then
       Update.commitUpdates ();
     display "No updates to propagate\n";
+    if skipped > 0 then begin
+      let summary =
+        Printf.sprintf
+          "Synchronization complete at %s  (0 item transferred, %d skipped, 0 failed)"
+          (let tm = Util.localtime (Util.time()) in
+           Printf.sprintf "%02d:%02d:%02d"
+                          tm.Unix.tm_hour tm.Unix.tm_min tm.Unix.tm_sec)
+          skipped in
+      Trace.log (summary ^ "\n");
+      Safelist.iter
+        (fun ri ->
+         match ri.replicas with
+           Problem r
+         | Different {rc1 = _; rc2 = _; direction = Conflict r; default_direction = _} ->
+            alwaysDisplayAndLog (Printf.sprintf "  skipped: %s (%s)"
+                                                (Path.toString ri.path1) r)
+         | _ -> ())
+        newReconItemList
+      end;
     (skipped > 0, false, false, [])
   end else if proceed=ProceedImmediately then begin
     doit()
@@ -613,9 +633,9 @@ let rec interactAndPropagateChanges reconItemList
     displayWhenInteractive "\nProceed with propagating updates? ";
     selectAction
       (* BCP: I find it counterintuitive that every other prompt except this one
-         would expect <CR> as a default.  But I got talked out of offering a default
-         here, because of safety considerations (too easy to press <CR> one time
-         too many). *)
+         would expect <CR> as a default.  But I got talked out of offering a
+         default here, because of safety considerations (too easy to press
+         <CR> one time too many). *)
       (if Prefs.read Globals.batch then Some "y" else None)
       [(["y";"g"],
         "Yes: proceed with updates as selected above",
@@ -656,7 +676,7 @@ let checkForDangerousPath dangerousPaths =
     end 
   end
 
-let synchronizeOnce() =
+let synchronizeOnce ?wantWatcher ?skipRecentFiles pathsOpt =
   let showStatus path =
     if path = "" then Util.set_infos "" else
     let max_len = 70 in
@@ -674,7 +694,10 @@ let synchronizeOnce() =
   if not (Prefs.read Trace.terse) && (Prefs.read Trace.debugmods = []) then
     Uutil.setUpdateStatusPrinter (Some showStatus);
 
-  let updates = Update.findUpdates() in
+  debug (fun() -> Util.msg "temp: Globals.paths = %s\n"
+           (String.concat " "
+              (Safelist.map Path.toString (Prefs.read Globals.paths))));
+  let updates = Update.findUpdates ?wantWatcher pathsOpt in
 
   Uutil.setUpdateStatusPrinter None;
   Util.set_infos "";
@@ -697,94 +720,79 @@ let synchronizeOnce() =
     (exitStatus, failedPaths)
   end
 
-let watchinterval = 10
+(* ----------------- Filesystem watching mode ---------------- *)
 
-(* FIX; Using string concatenation to accumulate characters is
-   pretty inefficient! *)
-let charsRead = ref ""
-let linesRead = ref []
-let watcherchan = ref None
+let watchinterval = 1.    (* Minimal interval between two synchronizations *)
+let retrydelay = 5.       (* Minimal delay to retry failed paths *)
+let maxdelay = 30. *. 60. (* Maximal delay to retry failed paths *)
 
-let suckOnWatcherFileLocal n =
-  Util.convertUnixErrorsToFatal
-    ("Reading changes from watcher process in file " ^
-     System.fspathToPrintString n)
-    (fun () ->
-       (* The main loop, invoked from two places below *)
-       let rec loop ch =
-         match try Some(input_char ch) with End_of_file -> None with
-           None ->
-             let res = !linesRead in
-             linesRead := [];
-             res
-         | Some(c) ->
-             if c = '\n' then begin
-               linesRead := !charsRead
-                            :: !linesRead;
-               charsRead := "";
-               loop ch
-             end else begin
-               charsRead := (!charsRead) ^ (String.make 1 c);
-               loop ch
-             end in
-       (* Make sure there's a file to watch, then read from it *)
-       match !watcherchan with
-         None -> 
-           if System.file_exists n then begin
-             let ch = System.open_in_bin n in
-             watcherchan := Some(ch);
-             loop ch
-           end else []
-       | Some(ch) -> loop ch
-      )
+module PathMap = Map.Make (Path)
 
-let suckOnWatcherFileRoot: Common.root -> System.fspath -> (string list) Lwt.t =
+let waitForChangesRoot: Common.root -> unit -> unit Lwt.t =
   Remote.registerRootCmd
-    "suckOnWatcherFile"
-    (fun (fspath, n) ->
-      Lwt.return (suckOnWatcherFileLocal n))
+    "waitForChanges"
+    (fun (fspath, _) -> Fswatchold.wait (Update.archiveHash fspath))
 
-let suckOnWatcherFiles n =
-  Safelist.concat
-    (Lwt_unix.run (
-      Globals.allRootsMap (fun r -> suckOnWatcherFileRoot r n)))
+let waitForChanges t =
+  let dt = t -. Unix.gettimeofday () in
+  if dt > 0. then begin
+    let timeout = if dt <= maxdelay then [Lwt_unix.sleep dt] else [] in
+    Lwt_unix.run
+      (Globals.allRootsMap (fun r -> Lwt.return (waitForChangesRoot r ()))
+         >>= fun l ->
+       Lwt.choose (timeout @ l))
+  end
 
 let synchronizePathsFromFilesystemWatcher () =
-  let watcherfilename = System.fspathFromString "" in
-  (* STOPPED HERE -- need to find the program using watcherosx preference
-     and invoke it (on both hosts, if there are two!) using a redirect to
-     get the output into a temp file... *)
-  let rec loop failedPaths = 
-    let newpaths = suckOnWatcherFiles watcherfilename in
-    if newpaths <> [] then
-      display (Printf.sprintf "Changed paths:\n  %s\n"
-                 (String.concat "\n  " newpaths));
-    let p = failedPaths @ (Safelist.map Path.fromString newpaths) in
-    if p <> [] then begin
-      Prefs.set Globals.paths p;
-      let (exitStatus,newFailedPaths) = synchronizeOnce() in 
-      debug (fun() -> Util.msg "Sleeping for %d seconds...\n" watchinterval);
-      Unix.sleep watchinterval;
-      loop newFailedPaths 
-    end else begin
-      debug (fun() -> Util.msg "Nothing changed: sleeping for %d seconds...\n"
-               watchinterval);
-      Unix.sleep watchinterval;
-      loop []
-    end in
-  loop []
+  let rec loop isStart delayInfo =
+    let t = Unix.gettimeofday () in
+    let (delayedPaths, readyPaths) =
+      PathMap.fold
+        (fun p (t', _) (delayed, ready) ->
+           if t' <= t then (delayed, p :: ready) else (p :: delayed, ready))
+        delayInfo ([], [])
+    in
+    let (exitStatus, failedPaths) =
+      synchronizeOnce ~wantWatcher:() ~skipRecentFiles:()
+        (if isStart then None else Some (readyPaths, delayedPaths))
+    in
+    (* After a failure, we retry at once, then use an exponential backoff *)
+    let delayInfo =
+      Safelist.fold_left
+        (fun newDelayInfo p ->
+           PathMap.add p
+             (try
+                let (t', d) = PathMap.find p delayInfo in
+                if t' > t then (t', d) else
+                let d = max retrydelay (min maxdelay (2. *. d)) in
+                (t +. d, d)
+              with Not_found ->
+                (t, 0.))
+             newDelayInfo)
+        PathMap.empty
+        (Safelist.append delayedPaths failedPaths)
+    in
+    Lwt_unix.run (Lwt_unix.sleep watchinterval);
+    let nextTime =
+      PathMap.fold (fun _ (t, d) t' -> min t t') delayInfo 1e20 in
+    waitForChanges nextTime;
+    loop false delayInfo
+  in
+  loop true PathMap.empty
 
-let synchronizeUntilNoFailures () =
-  let initValueOfPathsPreference = Prefs.read Globals.paths in
-  let rec loop triesLeft =
-    let (exitStatus,failedPaths) = synchronizeOnce() in
+(* ----------------- Repetition ---------------- *)
+
+let synchronizeUntilNoFailures repeatMode =
+  let rec loop triesLeft pathsOpt =
+    let (exitStatus, failedPaths) =
+      synchronizeOnce
+        ?wantWatcher:(if repeatMode then Some () else None) pathsOpt in
     if failedPaths <> [] && triesLeft <> 0 then begin
-      loop (triesLeft - 1)
+      loop (triesLeft - 1) (Some (failedPaths, []))
     end else begin
-      Prefs.set Globals.paths initValueOfPathsPreference;
       exitStatus
     end in
-  loop (Prefs.read Uicommon.retry)
+  loop (Prefs.read Uicommon.retry) None
 
 let rec synchronizeUntilDone () =
   let repeatinterval =
@@ -799,17 +807,30 @@ let rec synchronizeUntilDone () =
                            ^Prefs.read Uicommon.repeat
                            ^") should be either a number or 'watch'\n")) in
 
-  let exitStatus = synchronizeUntilNoFailures() in
+  let exitStatus = synchronizeUntilNoFailures(repeatinterval >= 0) in
   if repeatinterval < 0 then
     exitStatus
   else begin
     (* Do it again *)
-    Trace.status (Printf.sprintf "\nSleeping for %d seconds...\n" repeatinterval);
+    Trace.status (Printf.sprintf
+       "\nSleeping for %d seconds...\n" repeatinterval);
     Unix.sleep repeatinterval;
     synchronizeUntilDone ()
   end
 
-let start interface =
+(* ----------------- Startup ---------------- *)
+
+let handleException e =
+  restoreTerminal();
+  let msg = Uicommon.exn2string e in
+  Trace.log (msg ^ "\n");
+  if not !Trace.sendLogMsgsToStderr then begin
+    alwaysDisplay "\n";
+    alwaysDisplay msg;
+    alwaysDisplay "\n";
+  end
+
+let rec start interface =
   if interface <> Uicommon.Text then
     Util.msg "This Unison binary only provides the text GUI...\n";
   begin try
@@ -853,16 +874,26 @@ let start interface =
     exit exitStatus
 
   with
-    e ->
-      restoreTerminal();
-      let msg = Uicommon.exn2string e in
-      Trace.log (msg ^ "\n");
-      if not !Trace.sendLogMsgsToStderr then begin
-        alwaysDisplay "\n";
-        alwaysDisplay msg;
-        alwaysDisplay "\n";
-      end;
+    Sys.Break -> begin
+      (* If we've been killed, then die *)
+      handleException Sys.Break;
       exit Uicommon.fatalExit
+    end 
+  | e -> begin
+      (* If any other bad thing happened and the -repeat preference is
+         set, then restart *)
+      (* JV: it seems safer to just abort here, as we don't know in which
+         state Unison is; for instance, if the connection is lost, there
+         is no point in restarting as Unison will currently not attempt to
+         establish a new connection. *)
+      handleException e;
+      if false (*Prefs.read Uicommon.repeat <> ""*) then begin
+        Util.msg "Restarting in 10 seconds...\n";
+        Unix.sleep 10;        
+        start interface
+      end else
+        exit Uicommon.fatalExit
+    end 
   end
 
 let defaultUi = Uicommon.Text
